/***********************************************************************************************************************
 *
 * blueBill Mobile - open source birdwatching
 * ==========================================
 *
 * Copyright (C) 2009, 2010 by Tidalwave s.a.s. (http://www.tidalwave.it)
 * http://bluebill.tidalwave.it/mobile/
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * $Id: Taxon.java,v 0e8e9f7edc16 2010/06/18 23:18:02 fabrizio $
 *
 **********************************************************************************************************************/
package it.tidalwave.bluebill.taxonomy;

import javax.annotation.CheckForNull;
import javax.annotation.Nonnull;
import javax.annotation.Nullable;
import javax.annotation.concurrent.Immutable;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Locale;
import java.io.Serializable;
import it.tidalwave.util.Id;
import it.tidalwave.util.NotFoundException;
import it.tidalwave.bluebill.taxonomy.Taxonomy.Type;
import it.tidalwave.util.As;
import it.tidalwave.util.AsException;
import it.tidalwave.util.DefaultIdentifiable;
import it.tidalwave.util.Identifiable;

/***********************************************************************************************************************
 *
 * @author  Fabrizio Giudici
 * @version $Id: $
 *
 **********************************************************************************************************************/
@Immutable
public class Taxon implements As, Serializable
  {
    private static final long serialVersionUID = 2877367475785934L;

    public static final Class<Taxon> Taxon = Taxon.class;

    private final Taxonomy taxonomy;

    @Nonnull
    private final Id id;
    
    @Nonnull
    private final String scientificName;

    @Nonnull
    private final Type type;

    @CheckForNull
    private final Taxon parent;

    protected final List<Taxon> children = new ArrayList<Taxon>();

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    public Taxon (final @Nonnull Taxonomy taxonomy,
                  final @Nonnull Id id,
                  final @Nonnull String scientificName,
                  final @Nonnull Type type,
                  final @Nullable Taxon parent)
      {
        this.taxonomy = taxonomy;
        this.id = id;
        this.scientificName = scientificName;
        this.type = type;
        this.parent = parent;
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @Nonnull
    public Id getId()
      {
        return id;
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @Nonnull
    public String getScientificName()
      {
        return scientificName;
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @Nonnull
    public String getDisplayName (final @Nonnull Locale locale)
      throws NotFoundException
      {
        return taxonomy.getDisplayName(this, locale);
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @Nonnull
    public Taxon getParent()
      throws NotFoundException
      {
        return NotFoundException.throwWhenNull(parent, "No parent for " + this);
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @Nonnull
    public Type getType()
      {
        return type;
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @Nonnull
    public List<Taxon> getChildren()
      {
        return Collections.unmodifiableList(children);
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @Override @Nonnull
    public String toString()
      {
        return String.format("Taxon[%s]", scientificName);
      }

    public <T> T as (final @Nonnull Class<T> clazz)
      {
        if (Identifiable.class.equals(clazz))
          {
            return clazz.cast(new DefaultIdentifiable(id));
          }

        throw new AsException(clazz);
      }

    public <T> T as(Class<T> clazz, NotFoundBehaviour<T> notFoundBehaviour)
      {
        throw new UnsupportedOperationException("Not supported yet.");
      }
  }
